package org.implab.gradle.containers.tasks;

import java.io.File;
import java.util.Optional;

import org.gradle.api.DefaultTask;
import org.gradle.api.logging.Logger;
import org.gradle.api.provider.Property;
import org.gradle.api.tasks.Input;
import org.gradle.api.tasks.Internal;
import org.implab.gradle.containers.ContainerExtension;
import org.implab.gradle.containers.PropertiesMixin;
import org.implab.gradle.containers.cli.DockerTraits;
import org.implab.gradle.containers.cli.ProcessSpec;
import org.implab.gradle.containers.cli.RedirectFrom;
import org.implab.gradle.containers.cli.RedirectTo;

public abstract class DockerCliTask extends DefaultTask implements PropertiesMixin {

    @Input
    public abstract Property<String> getCliCmd();

    /**
     * Returns working directory for docker commands
     */
    @Input
    @org.gradle.api.tasks.Optional
    public abstract Property<File> getWorkingDirectory();

    @Internal
    protected ContainerExtension getContainerExtension() {
        return getProject().getExtensions().getByType(ContainerExtension.class);
    }

    public DockerCliTask() {
        getCliCmd().convention(getContainerExtension().getCliCmd());

    }

    protected Optional<RedirectTo> loggerInfoRedirect() {
        return getLogger().isInfoEnabled() ? Optional.of(RedirectTo.consumer(getLogger()::info)) : Optional.empty();
    }

    protected Optional<RedirectTo> loggerErrorRedirect() {
        return getLogger().isErrorEnabled() ? Optional.of(RedirectTo.consumer(getLogger()::error)) : Optional.empty();
    }

    protected Optional<RedirectTo> getStdoutRedirection() {
        return loggerInfoRedirect();
    }

    protected Optional<RedirectTo> getStderrRedirection() {
        return loggerErrorRedirect();
    }

    protected Optional<RedirectFrom> getStdinRedirection() {
        return Optional.empty();
    }

    protected DockerTraits docker() {
        return new TaskDockerTraits();
    }

    protected void exec(ProcessSpec spec) {

        getLogger().info("Starting: {}", spec.command());
    }

    class TaskDockerTraits extends DockerTraits {

        @Override
        public Logger getLogger() {
            return DockerCliTask.this.getLogger();
        }

        @Override
        public Optional<File> getWorkingDir() {
            return getWorkingDirectory()
                    .map(Optional::of)
                    .getOrElse(Optional.empty());
        }

        @Override
        public String getCliCmd() {
            return DockerCliTask.this.getCliCmd().get();
        }

    }
}
